@@ -9,6 +9,7 @@ import threading
 import typing
 import uuid
 from enum import Enum
+from typing import Any
 
 import duckdb
 import pandas as pd
@@ -154,9 +155,20 @@ class Cursor:
                         "session_id": self.session_id,  # type: ignore
                     }
                 )
+
                 new_row_dict.update(key_values)
-                new_row_df = pd.DataFrame(new_row_dict, index=[0])
-                new_row = pa.Table.from_pandas(new_row_df, schema=table.schema)
+                new_row_df = pd.DataFrame([new_row_dict])
+
+                try:
+                    new_row = pa.Table.from_pandas(new_row_df, schema=table.schema)
+                except pa.ArrowInvalid as e:
+                    raise TypeError(
+                        f"Invalid key-value pair for relation {relation}. Make sure the values are of the correct type. Full error: {e}"
+                    )
+                except pa.ArrowTypeError as e:
+                    raise TypeError(
+                        f"Invalid key-value pair for relation {relation}. Make sure the values are of the correct type. Full error: {e}"
+                    )
 
                 # Check schemas match
                 if collections.Counter(new_row.schema.names) != collections.Counter(
@@ -218,6 +230,7 @@ class Cursor:
         trigger_name: str,
         trigger_version: int,
         trigger_action: str,
+        trigger_action_type: str,
         trigger_context: TriggerElement,
     ) -> None:
         """Logs the execution of a trigger.
@@ -225,7 +238,8 @@ class Cursor:
         Args:
             trigger_name (str): The name of the trigger.
             trigger_version (int): The version of the trigger.
-            trigger_action (str): The action of the trigger.
+            trigger_action (str): The action of the trigger (method name).
+            trigger_action_type (str): The type of action (infer or fit).
             trigger_context (TriggerElement): The element that triggered the trigger.
         """
 
@@ -236,16 +250,13 @@ class Cursor:
             "trigger_name": trigger_name,
             "trigger_version": trigger_version,
             "trigger_action": trigger_action,
+            "trigger_action_type": trigger_action_type,
             "relation": trigger_context.relation,
             "identifier": trigger_context.identifier,
             "trigger_key": trigger_context.key,
         }
-        new_row = pa.Table.from_pandas(
-            pd.DataFrame(new_row, index=[0]), schema=self.log_table.schema
-        )
-
         with self.write_lock:
-            self.log_table = pa.concat_tables([self.log_table, new_row])
+            self.log_table.loc[len(self.log_table)] = pd.Series(new_row)
 
     def executeTrigger(
         self,
@@ -312,7 +323,11 @@ class Cursor:
         if route.infer is not None:
             infer_context = route.infer(new_connection, trigger_context)
             self.logTriggerExecution(
-                trigger_name, trigger_fn.version, "infer", trigger_context
+                trigger_name,
+                trigger_fn.version,
+                route.infer.__name__,
+                "INFER",
+                trigger_context,
             )
 
         # Fit is asynchronous
@@ -459,9 +474,7 @@ class Cursor:
             **kwargs,
         )
 
-    def _get_derived_ids(
-        self, con: duckdb.DuckDBPyConnection, identifier: str
-    ) -> list[str]:
+    def _get_derived_ids(self, con: duckdb.DuckDBPyConnection, identifier: str) -> Any:
         """Get all derived ids for an identifier.
 
         Args:
@@ -555,7 +568,7 @@ class Cursor:
 
         return res.to_dict("records")
 
-    def getIdsForKey(self, relation: str, key: str, value: typing.Any) -> list[int]:
+    def getIdsForKey(self, relation: str, key: str, value: Any) -> Any:
         """Get ids for a key-value pair in a relation.
 
         Args:
